// Copyright 2015 Google Inc. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package samples

import (
	"fmt"
	"io/ioutil"
	"time"

	"github.com/googlecloudplatform/gcsfuse/timeutil"
	"github.com/jacobsa/fuse"
)

// A struct that implements common behavior needed by tests in the samples/
// directory. Use it as an anonymous member of your test fixture, calling its
// Initialize method from your SetUp method and its Destroy method from your
// TearDown method.
type SampleTest struct {
	// A clock with a fixed initial time. The test's set up method may use this
	// to wire the file system with a clock, if desired.
	Clock timeutil.SimulatedClock

	// The directory at which the file system is mounted.
	Dir string

	mfs *fuse.MountedFileSystem
}

// Mount the supplied file system and initialize the exported fields of the
// struct. Panics on error.
func (t *SampleTest) Initialize(fs fuse.FileSystem, config *fuse.MountConfig) {
	err := t.initialize(fs, config)
	if err != nil {
		panic(err)
	}
}

// Like Initialize, but doens't panic.
func (t *SampleTest) initialize(
	fs fuse.FileSystem,
	config *fuse.MountConfig) (err error) {
	// Initialize the clock.
	t.Clock.SetTime(time.Date(2012, 8, 15, 22, 56, 0, 0, time.Local))

	// Set up a temporary directory.
	t.Dir, err = ioutil.TempDir("", "sample_test")
	if err != nil {
		err = fmt.Errorf("TempDir: %v", err)
		return
	}

	// Mount the file system.
	t.mfs, err = fuse.Mount(t.Dir, fs, config)
	if err != nil {
		err = fmt.Errorf("Mount: %v", err)
		return
	}

	return
}

// Unmount the file system and clean up. Panics on error.
func (t *SampleTest) Destroy()
